from gocd_tools.cli.defaults import SECRETS, SERVER
from gocd_tools.cli.run import cli_exec
from gocd_tools.cli.secrets import add_secrets_groups
from gocd_tools.cli.server import add_server_groups


def setup_cli(parser):
    setup_commands = parser.add_subparsers(title="COMMAND")

    # Secrets
    secrets_parser = setup_commands.add_parser(SECRETS)
    add_secrets_groups(secrets_parser)
    secrets_commands = secrets_parser.add_subparsers(title="COMMAND")
    secrets_init_parser = secrets_commands.add_parser(
        "init",
        help=(
            "Initializes the secrets db directory where the GoCD secrets will be stored."
        ),
    )
    secrets_init_parser.set_defaults(
        func=cli_exec,
        module_path="gocd_tools.secrets",
        module_name="secrets",
        func_name="init_secrets_dir",
    )

    secrets_configure_parser = secrets_commands.add_parser(
        "configure",
        help=(
            "Configure the GoCD server with the secrets defined in the secrets"
            " db directory."
        ),
    )
    secrets_configure_parser.set_defaults(
        func=cli_exec,
        module_path="gocd_tools.secrets",
        module_name="secrets",
        func_name="configure_secrets",
    )

    secrets_del_parser = secrets_commands.add_parser(
        "delete", help="Delete the secrets db directory."
    )
    secrets_del_parser.set_defaults(
        func=cli_exec,
        module_path="gocd_tools.secrets",
        module_name="secrets",
        func_name="del_secrets_dir",
    )

    # Server
    server_parsers = setup_commands.add_parser(SERVER)
    add_server_groups(server_parsers)
    server_commands = server_parsers.add_subparsers(title="COMMAND")
    server_init_parser = server_commands.add_parser(
        "init",
        help=(
            "Initializes the server with an Authorization Configiguration as"
            " defined in the local config directory's authorization_config.yml"
            " file."
        ),
    )
    server_init_parser.set_defaults(
        func=cli_exec,
        module_path="gocd_tools.server",
        module_name="server",
        func_name="init_server",
    )

    server_configure_parser = server_commands.add_parser(
        "configure",
        help=(
            "Configures the designated GoCD server with the configuration"
            " files stored in the local config directory."
        ),
    )
    server_configure_parser.set_defaults(
        func=cli_exec,
        module_path="gocd_tools.server",
        module_name="server",
        func_name="configure_server",
    )

    server_cleanup_parser = server_commands.add_parser(
        "wipe",
        help=(
            "Will wipe all the configuration settings defined in the local"
            " config directory on the GoCD server."
        ),
    )
    server_cleanup_parser.set_defaults(
        func=cli_exec,
        module_path="gocd_tools.server",
        module_name="server",
        func_name="wipe_server",
    )



import argparse
import datetime
import json
from gocd_tools.defaults import PACKAGE_NAME
from gocd_tools.utils import eprint
from gocd_tools.cli.setup import setup_cli


def to_str(o):
    if isinstance(o, datetime.datetime):
        return o.__str__()


def run():
    parser = argparse.ArgumentParser(prog=PACKAGE_NAME)
    commands = parser.add_subparsers(title="COMMAND")
    setup_parser = commands.add_parser("setup")
    setup_cli(setup_parser)

    args = parser.parse_args()
    # Execute default function
    if hasattr(args, "func"):
        success, response = args.func(args)
        output = ""
        if success:
            response["status"] = "success"
        else:
            response["status"] = "failed"

        try:
            output = json.dumps(response, indent=4, sort_keys=True, default=to_str)
        except Exception as err:
            eprint("Failed to format: {}, err: {}".format(output, err))
        if success:
            print(output)
        else:
            eprint(output)
    return None



import os


def makedirs(path):
    try:
        os.makedirs(os.path.expanduser(path))
        return True, "Created: {}".format(path)
    except Exception as err:
        return False, "Failed to create the directory path: {} - {}".format(path, err)
    return False, "Failed to create the directory path: {}".format(path)


def load(path, mode="r", readlines=False, handler=None, **load_kwargs):
    try:
        with open(path, mode) as fh:
            if handler:
                return handler.load(fh, **load_kwargs)
            if readlines:
                return fh.readlines()
            return fh.read()
    except Exception as err:
        print("Failed to load file: {} - {}".format(path, err))
    return False


def write(path, content, mode="w", mkdirs=False, handler=None, **handler_kwargs):
    dir_path = os.path.dirname(path)
    if not os.path.exists(dir_path) and mkdirs:
        if not makedirs(dir_path):
            return False
    try:
        with open(path, mode) as fh:
            if handler:
                handler.dump(content, fh, **handler_kwargs)
            else:
                fh.write(content)
        return True
    except Exception as err:
        print("Failed to save file: {} - {}".format(path, err))
    return False


def remove(path):
    try:
        if exists(path):
            os.remove(os.path.expanduser(path))
            return True, "Removed file: {}".format(path)
    except Exception as err:
        return False, "Failed to remove file: {} - {}".format(path, err)
    return False, "Failed to remove file: {}".format(path)


def removedir(path):
    try:
        if exists(path):
            os.rmdir(os.path.expanduser(path))
            return True, "Removed directory: {}".format(path)
    except Exception as err:
        return False, "Failed to remove directory: {} - {}".format(path, err)
    return False, "Failed to remove directory: {}".format(path)


def exists(path):
    return os.path.exists(os.path.expanduser(path))


def chmod(path, mode, **kwargs):
    try:
        os.chmod(os.path.expanduser(path), mode, **kwargs)
    except Exception as err:
        return (
            False,
            "Failed to set permissions: {} on: {} - {}".format(mode, path, err),
        )
    return True, "Set the path: {} with permissions: {}".format(path, mode)



import yaml
from gocd_tools.io import exists, load


def load_config(path=None):
    if not config_exists(path):
        return False
    return load(path, handler=yaml, Loader=yaml.FullLoader)


def config_exists(path):
    if not path:
        return False
    return exists(path)



import os
from gocd_tools.utils import is_env_set

PACKAGE_NAME = "gocd-tools"

# API Request Defaults
CONTENT_TYPE = "application/json"
JSON_HEADER = {"Content-Type": CONTENT_TYPE}

API_VERSION_1 = "application/vnd.go.cd.v1+json"
API_VERSION_2 = "application/vnd.go.cd.v2+json"
API_VERSION_3 = "application/vnd.go.cd.v3+json"
API_VERSION_4 = "application/vnd.go.cd.v4+json"
API_VERSION_7 = "application/vnd.go.cd.v7+json"

ACCEPT_HEADER_1 = {"Accept": API_VERSION_1, **JSON_HEADER}
ACCEPT_HEADER_2 = {"Accept": API_VERSION_2, **JSON_HEADER}
ACCEPT_HEADER_3 = {"Accept": API_VERSION_3, **JSON_HEADER}
ACCEPT_HEADER_4 = {"Accept": API_VERSION_4, **JSON_HEADER}
ACCEPT_HEADER_7 = {"Accept": API_VERSION_7, **JSON_HEADER}

GITHUB_API_VERSION = "application/vnd.github.v3+json"
GITHUB_ACCEPT_HEADER = {"Accept": GITHUB_API_VERSION}

# API default endpoints
if "GOCD_BASE_URL" in os.environ:
    GOCD_BASE_URL = os.environ["GOCD_BASE_URL"]

if GOCD_BASE_URL == "":
    print(
        "The require environment variable GOCD_BASE_URL was empty: {}".format(
            GOCD_BASE_URL
        )
    )
    exit(1)

# Github URLs
GITHUB_AUTH_URL = "https://api.github.com/user"
GITHUB_GOCD_AUTH_URL = ""

# Public URLs
GO_URL = "{}/go".format(GOCD_BASE_URL)
API_URL = "{}/api".format(GO_URL)
AUTH_URL = "{}/current_user".format(API_URL)
ELASTIC_AGENT_URL = "{}/elastic/profiles".format(API_URL)
ADMIN_URL = "{}/admin".format(API_URL)
SECURITY_URL = "{}/security".format(ADMIN_URL)
ROLE_URL = "{}/roles".format(SECURITY_URL)

# Restricted URLs
AUTHORIZATION_CONFIG_URL = "{}/auth_configs".format(SECURITY_URL)
CLUSTER_PROFILES_URL = "{}/elastic/cluster_profiles".format(ADMIN_URL)
PIPELINE_GROUPS_URL = "{}/pipeline_groups".format(ADMIN_URL)
CONFIG_REPO_URL = "{}/config_repos".format(ADMIN_URL)
TEMPLATE_URL = "{}/templates".format(ADMIN_URL)
SECRET_CONFIG_URL = "{}/secret_configs".format(ADMIN_URL)

# Server config
CONFIG_SERVER = "{}/config/server".format(ADMIN_URL)
ARTIFACT_CONFIG = "{}/artifact_config".format(CONFIG_SERVER)

if "GOCD_AUTH_TOKEN" in os.environ:
    GOCD_AUTH_TOKEN = os.environ["GOCD_AUTH_TOKEN"]
    # The GOCD_AUTH_TOKEN is the one generate within the GOCD server
    # (Not GitHub)
else:
    GOCD_AUTH_TOKEN = ""

if "GITHUB_AUTH_TOKEN" in os.environ:
    GITHUB_AUTH_TOKEN = os.environ["GITHUB_AUTH_TOKEN"]
else:
    GITHUB_AUTH_TOKEN = ""


AUTHORIZATION_HEADER = {
    "Authorization": "bearer {}".format(GOCD_AUTH_TOKEN),
    **ACCEPT_HEADER_1,
}

GITHUB_AUTHORIZATION_HEADER = {
    "Authorization": "token {}".format(GITHUB_AUTH_TOKEN),
    **GITHUB_ACCEPT_HEADER,
}

# Server defaults
GO_DATA_DIR = "/godata"
GO_SECRET_DIR = "/gosecret"
GO_SECRET_DB_FILE = "secrets.yml"
GO_CONFIG_DIR = os.path.join(os.path.expanduser("~"), ".{}".format(PACKAGE_NAME))
GO_PLUGIN_DIR = os.path.join(GO_DATA_DIR, "plugins")

# Other constants
GOCD_SECRET_PLUGIN = "gocd-file-based-secrets-plugin.jar"

BUNDLED_PLUGIN = "bundled"
EXTERNAL_PLUGIN = "external"

# Environment variables
ENV_GO_DATA_DIR = "GO_DATA_DIR"
ENV_GO_SECRET_DIR = "GO_SECRET_DIR"
ENV_GO_SECRET_DB_FILE = "GO_SECRET_DB_FILE"
ENV_GO_CONFIG_DIR = "GO_CONFIG_DIR"
ENV_GO_PLUGIN_DIR = "GO_PLUGIN_DIR"

# Default configuration input paths
default_base_path = GO_CONFIG_DIR
default_config_path = os.path.join(default_base_path, "config")

artifacts_config_path = os.path.join(default_config_path, "artifacts_config.yml")
authorization_config_path = os.path.join(
    default_config_path, "authorization_configuration.yml"
)
cluster_profiles_path = os.path.join(default_config_path, "cluster_profiles.yml")
config_repositories_path = os.path.join(default_config_path, "config_repositories.yml")
elastic_agent_profile_path = os.path.join(
    default_config_path, "elastic_agent_profiles.yml"
)
pipeline_group_configs_path = os.path.join(
    default_config_path, "pipeline_group_configs.yml"
)
roles_path = os.path.join(default_config_path, "roles.yml")
secret_configs_path = os.path.join(default_config_path, "secret_configs.yml")
templates_path = os.path.join(default_config_path, "templates.yml")


# Datadir discover
def get_data_dir():
    data_path, msg = is_env_set(ENV_GO_DATA_DIR)
    if not data_path:
        if GO_DATA_DIR:
            return GO_DATA_DIR, ""
        return False, msg
    return data_path, ""


# Secrets discover
def get_secrets_dir_path():
    env_dir_path, msg = is_env_set(ENV_GO_SECRET_DIR)
    if not env_dir_path:
        # Use the default as a backup
        if GO_SECRET_DIR:
            return GO_SECRET_DIR, ""
        return False, msg
    return env_dir_path, ""


def get_secrets_file_name():
    secrets_name, msg = is_env_set(ENV_GO_SECRET_DB_FILE)
    if not secrets_name:
        # Use the default as a backup
        if GO_SECRET_DB_FILE:
            return GO_SECRET_DB_FILE, ""
        return False, msg
    return secrets_name, ""


def get_secrets_db_path():
    dir_path, msg = get_secrets_dir_path()
    if not dir_path:
        return False, msg
    file_name, msg = get_secrets_file_name()
    if not file_name:
        return False, msg
    return os.path.join(dir_path, file_name), ""



import os
from gocd_tools.defaults import (
    is_env_set,
    ENV_GO_PLUGIN_DIR,
    GO_PLUGIN_DIR,
    BUNDLED_PLUGIN,
    GOCD_SECRET_PLUGIN,
)


def get_plugin_dir():
    plugin_path, msg = is_env_set(ENV_GO_PLUGIN_DIR)
    if not plugin_path:
        if GO_PLUGIN_DIR:
            return GO_PLUGIN_DIR, ""
        return False, msg
    return plugin_path, ""


def get_plugin_path(plugin_type=BUNDLED_PLUGIN, plugin=GOCD_SECRET_PLUGIN):
    plugin_dir_path, msg = get_plugin_dir()
    if not plugin_dir_path:
        return False, msg
    return os.path.join(plugin_dir_path, plugin_type, plugin), ""



import requests
import json
from gocd_tools.defaults import (
    authorization_config_path,
    artifacts_config_path,
    cluster_profiles_path,
    elastic_agent_profile_path,
    pipeline_group_configs_path,
    config_repositories_path,
    roles_path,
    templates_path,
    secret_configs_path,
    GITHUB_GOCD_AUTH_URL,
    ACCEPT_HEADER_1,
    ACCEPT_HEADER_2,
    ACCEPT_HEADER_3,
    ACCEPT_HEADER_4,
    ACCEPT_HEADER_7,
    GOCD_BASE_URL,
    AUTH_URL,
    AUTHORIZATION_HEADER,
    SECRET_CONFIG_URL,
    AUTHORIZATION_CONFIG_URL,
    CLUSTER_PROFILES_URL,
    ELASTIC_AGENT_URL,
    PIPELINE_GROUPS_URL,
    ROLE_URL,
    CONFIG_REPO_URL,
    ARTIFACT_CONFIG,
    TEMPLATE_URL,
)
from gocd_tools.config import load_config


def head(session, url, *args, **kwargs):
    try:
        return session.head(url, *args, **kwargs)
    except Exception as err:
        print("Failed HEAD request: {}".format(err))
    return None


def get(session, url, *args, **kwargs):
    try:
        return session.get(url, *args, **kwargs)
    except Exception as err:
        print("Failed GET request: {}".format(err))
    return None


def delete(session, url, *args, **kwargs):
    try:
        return session.delete(url, *args, **kwargs)
    except Exception as err:
        print("Failed DELETE request: {}".format(err))
    return None


def post(session, url, *args, **kwargs):
    try:
        return session.post(url, *args, **kwargs)
    except Exception as err:
        print("Failed POST request: {}".format(err))
    return None


def put(session, url, *args, **kwargs):
    try:
        return session.put(url, *args, **kwargs)
    except Exception as err:
        print("Failed PUT request: {}".format(err))
    return None


def get_type(session, base_url, id, headers=None):
    id_url = "{}/{}".format(base_url, id)
    resp = get(session, id_url, headers=headers)
    if resp.status_code == 200:
        return resp.headers, resp.text
    print("Failed to find: {}:{}".format(resp.status_code, resp.text))
    return None, None


def get_types(session, base_url, headers=None):
    resp = get(session, base_url, headers=headers)
    if resp.status_code == 200:
        return resp.headers, resp.text
    print("{}:{}".format(resp.status_code, resp.text))
    return None, None


def create_type(session, url, data=None, headers=None):
    json_data = json.dumps(data)
    created = post(session, url, data=json_data, headers=headers)
    if created.status_code == 200:
        return True
    print("{}:{}".format(created.status_code, created.text))
    return False


def delete_type(session, base_url, id, headers=None):
    id_url = "{}/{}".format(base_url, id)
    created = delete(session, id_url, headers=headers)
    if created.status_code == 200:
        return True
    print("{}:{}".format(created.status_code, created.text))
    return False


def update(session, url, data=None, headers=None):
    json_data = json.dumps(data)
    updated = put(session, url, data=json_data, headers=headers)
    if updated.status_code == 200 or updated.status_code == 201:
        return True
    print("Failed to update type: {}:{}".format(updated.status_code, updated.text))
    return False


def update_type(session, base_url, id, data=None, headers=None):
    id_url = "{}/{}".format(base_url, id)
    return update(session, id_url, data=data, headers=headers)


# def authenticate_github(session):
#     if GOCD_AUTH_TOKEN == "":
#         print(
#             "The required environment variable GOCD_AUTH_TOKEN was empty: {}".format(
#                 GOCD_AUTH_TOKEN
#             )
#         )
#         return False

#     # Login to regular github
#     github_auth = get(session, GITHUB_AUTH_URL, headers=GITHUB_AUTHORIZATION_HEADER)
#     if github_auth.status_code != 200:
#         print(
#             "Failed to authenticate with GitHub: {} - {}".format(
#                 github_auth.status_code, github_auth.text
#             )
#         )
#         return False

#     redirect_location = get(
#         session,
#         GITHUB_GOCD_AUTH_URL,
#         allow_redirects=False,
#         headers=GITHUB_AUTHORIZATION_HEADER,
#     )
#     if not redirect_location.status_code == 302:
#         print(
#             "Failed to get the GitHub auth location: {} - {}".format(
#                 redirect_location.status_code, redirect_location.text
#             )
#         )
#         return False

#     if "location" not in redirect_location.headers:
#         print(
#             "Location not available in the GitHUB auth response: {}".format(
#                 redirect_location.headers
#             )
#         )
#         return False

#     # Get the URL location for the GitHub auth page.
#     # TODO, the location_url does not properly authorize
#     # the session with the existing github authentication.
#     # Potentially lookinto a library option like
#     # https://requests-oauthlib.readthedocs.io/en/latest/oauth2_workflow.html
#     # location_url = redirect_location.headers["location"]
#     # github_resp = get(
#     #    session, location_url, allow_redirects=False
#     # )
#     #    if not github_resp.status_code == 200:
#     #        return False
#     #    return True
#     return False


def authenticate(session, **auth_kwargs):
    resp = get(session, AUTH_URL, headers=AUTHORIZATION_HEADER, **auth_kwargs)
    if resp.status_code == 200:
        return True
    print("Failed to authenticate: {} - {}".format(resp.status_code, resp.text))
    return False


def is_auth_repo(repository_config):
    if "authentication" not in repository_config:
        return False
    if "required" not in repository_config["authentication"]:
        return False
    if not repository_config["authentication"]["required"]:
        return False
    return True


def repo_auth_data(repository_config):
    return repository_config["authentication"]


def get_repo_secret(auth_data):
    if "secret" not in auth_data:
        return False
    return auth_data["secret"]


def get_repo_secret_manager(repository_config):
    return repository_config["authentication"]["secret_plugin"]


def get_secret_configs(session, headers=None):
    if not headers:
        headers = ACCEPT_HEADER_3
    return get(session, SECRET_CONFIG_URL)


def get_secret_manager(session, id):
    return get_type(session, SECRET_CONFIG_URL, id, headers=ACCEPT_HEADER_3)


def get_aritifacts_config(session, url, headers):
    resp = get(session, url, headers=headers)
    if resp.status_code == 200:
        return resp.headers, resp.text
    return None, None


def init_server():
    """As the first thing, the server's authorization config
    is defined.
    """
    print("Init server: {}".format(GOCD_BASE_URL))
    authorization_configs = load_config(path=authorization_config_path)

    response = {}
    with requests.Session() as session:
        for auth_config in authorization_configs:
            exists = get_type(
                session,
                AUTHORIZATION_CONFIG_URL,
                auth_config["id"],
                headers=ACCEPT_HEADER_2,
            )
            if not exists:
                created = create_type(
                    session,
                    AUTHORIZATION_CONFIG_URL,
                    data=auth_config,
                    headers=ACCEPT_HEADER_2,
                )
                if not created:
                    response[
                        "msg"
                    ] = "Failed to create authorization config: {}".format(auth_config)
                    return False, response
    response["msg"] = "The Authorization config for: {} was completed".format(
        GOCD_BASE_URL
    )
    return True, response


def update_artifacts_config(session, config, url, headers):
    response = {}
    resp_headers, exists = get_aritifacts_config(session, url, headers)
    if not exists:
        response["msg"] = "Failed to find artifacts config to update"
        return False, response

    update_headers = {"If-Match": resp_headers["ETag"], **headers}
    updated = update(
        session,
        url,
        data=config,
        headers=update_headers,
    )
    if not updated:
        response["msg"] = "Failed to update: {}".format(config)
        return False, response
    return True, response


def setup_config(session, config, url, headers, identifer_variable="id"):
    response = {}
    resp_headers, exists = get_type(
        session,
        url,
        config[identifer_variable],
        headers=headers,
    )
    if not exists:
        print("Creating: {}".format(config[identifer_variable]))
        created = create_type(session, url, data=config, headers=headers)
        if not created:
            response["msg"] = "Failed to create: {}".format(config)
            return False, response
    else:
        update_headers = {"If-Match": resp_headers["ETag"], **headers}
        updated = update_type(
            session,
            url,
            config[identifer_variable],
            data=config,
            headers=update_headers,
        )
        if not updated:
            response["msg"] = "Failed to update: {}".format(config)
            return False, response
    return True, response


def setup_configs(session, configs, url, headers, identifer_variable="id"):
    for config in configs:
        success, response = setup_config(
            session,
            config,
            url,
            headers,
            identifer_variable=identifer_variable,
        )
        if not success:
            return False, response
    return True, {}


def remove_config(session, config, url, headers, identifier_variable="id"):
    response = {}
    resp_headers, exists = get_type(
        session,
        url,
        config[identifier_variable],
        headers=headers,
    )
    if exists:
        print("Removing: {}".format(config[identifier_variable]))
        deleted = delete_type(
            session, url, config[identifier_variable], headers=headers
        )
        if not deleted:
            response["msg"] = "Failed to remove: {}".format(config)
            return False, response
    return True, response


def remove_configs(session, configs, url, headers, identifier_variable="id"):
    for config in configs:
        success, response = remove_config(
            session,
            config,
            url,
            headers,
            identifier_variable=identifier_variable,
        )
        if not success:
            return False, response
    return True, {}


def setup_config_repositories(session, configs, url, headers):
    response = {}
    for repository_config in configs:
        resp_headers, exists = get_type(
            session,
            url,
            repository_config["id"],
            headers=headers,
        )
        if not exists:
            # Check whether a secret auth token is required
            if is_auth_repo(repository_config):
                auth_data = repo_auth_data(repository_config)
                repo_secret_manager = get_repo_secret_manager(repository_config)
                secret = get_repo_secret(auth_data)

                secret_manager = get_secret_manager(session, repo_secret_manager)
                if not secret_manager:
                    print(
                        "Repo: {} tries to use secret manager: {}"
                        " which doesn't exist".format(
                            repository_config["id"], repo_secret_manager
                        )
                    )
                # secret = get_type(session, SECRET_CONFIG_URL)
                # if not secret:
                # created = create_secret(repository_config)
                #    created = create_secret(repository_config)
                #    if not created:
                #        print("Failed to create new secret")
                #    extra_config_kwargs["secret"] = created
            print("Creating: {}".format(repository_config["id"]))
            created = create_type(
                session,
                url,
                data={
                    "id": repository_config["id"],
                    **repository_config["config"],
                },
                headers=headers,
            )
            if not created:
                response["msg"] = "Failed to create repository config: {}".format(
                    created
                )
                return False, response
        else:
            updated = update_type(
                session,
                url,
                repository_config["id"],
                data=repository_config,
                headers=headers,
            )
            if not updated:
                response["msg"] = "Failed to update repository config: {}".format(
                    repository_config
                )
                return False, response
    return True, {}


def configure_server():
    authorization_configs = load_config(path=authorization_config_path)
    artifacts_config = load_config(path=artifacts_config_path)
    cluster_profiles_configs = load_config(path=cluster_profiles_path)
    elastic_agent_configs = load_config(path=elastic_agent_profile_path)
    pipeline_group_configs = load_config(path=pipeline_group_configs_path)
    repositories_configs = load_config(path=config_repositories_path)
    roles_configs = load_config(path=roles_path)
    templates_configs = load_config(path=templates_path)
    # TODO, load and create the authorization config
    secret_configs = load_config(path=secret_configs_path)

    configs = [
        {"path": authorization_config_path, "config": authorization_configs},
        {"path": artifacts_config_path, "config": artifacts_config},
        {"path": cluster_profiles_path, "config": cluster_profiles_configs},
        {"path": elastic_agent_profile_path, "config": elastic_agent_configs},
        {
            "path": pipeline_group_configs_path,
            "config": pipeline_group_configs,
        },
        {"path": config_repositories_path, "config": repositories_configs},
        {"path": roles_path, "config": roles_configs},
        {"path": templates_path, "config": templates_configs},
        {
            "path": secret_configs_path,
            "config": secret_configs,
        },
    ]
    response = {}

    for config in configs:
        if not config["config"]:
            response["msg"] = "Failed loading: {} - config: {}".format(
                config["path"], config["config"]
            )
            return False, response

    with requests.Session() as session:
        print("Authenticate")
        # github_authed = authenticate_github(session)
        # if not github_authed:
        #    response["msg"] = "Failed to authenticate against: {}".format(
        #        GITHUB_AUTH_URL
        #    )
        #    return False, response
        authed = authenticate(session)
        if not authed:
            response["msg"] = "Failed to authenticate against: {}".format(
                GITHUB_GOCD_AUTH_URL
            )
            return False, response

        print("Setup Roles")
        success, response = setup_configs(
            session,
            roles_configs,
            ROLE_URL,
            ACCEPT_HEADER_3,
            identifer_variable="name",
        )
        if not success:
            return False, response

    with requests.Session() as session:
        print("Re-Authenticate")
        authed = authenticate(session)
        if not authed:
            response["msg"] = "Failed to authenticate against: {}".format(
                GITHUB_GOCD_AUTH_URL
            )
            return False, response

        if artifacts_config:
            print("Setup Artifacts Config")
            success, response = update_artifacts_config(
                session, artifacts_config, ARTIFACT_CONFIG, ACCEPT_HEADER_1
            )
            if not success:
                return False, response

        if secret_configs:
            print("Setup Secret Configs")
            success, response = setup_configs(
                session,
                secret_configs,
                SECRET_CONFIG_URL,
                ACCEPT_HEADER_3,
            )
            if not success:
                return False, response

        if cluster_profiles_configs:
            print("Setup Cluster Profiles")
            success, response = setup_configs(
                session,
                cluster_profiles_configs,
                CLUSTER_PROFILES_URL,
                ACCEPT_HEADER_1,
            )
            if not success:
                return False, response

        if elastic_agent_configs:
            print("Setup Elastic Agent Configs")
            success, response = setup_configs(
                session,
                elastic_agent_configs,
                ELASTIC_AGENT_URL,
                ACCEPT_HEADER_2,
            )
            if not success:
                return False, response

        if pipeline_group_configs:
            print("Setup Pipeline Group Configs")
            success, response = setup_configs(
                session,
                pipeline_group_configs,
                PIPELINE_GROUPS_URL,
                ACCEPT_HEADER_1,
                identifer_variable="name",
            )
            if not success:
                return False, response

        if templates_configs:
            print("Create Template Configs")
            success, response = setup_configs(
                session,
                templates_configs,
                TEMPLATE_URL,
                ACCEPT_HEADER_7,
                identifer_variable="name",
            )
            if not success:
                return False, response

        if repositories_configs:
            print("Create Config Repositories")
            success, response = setup_config_repositories(
                session, repositories_configs, CONFIG_REPO_URL, ACCEPT_HEADER_4
            )

            if not success:
                return False, response

        if "msg" not in response:
            response["msg"] = "Succesfully configured the {} endpoint".format(
                GOCD_BASE_URL
            )
        return True, response

    return None, response


def wipe_server():
    repositories_configs = load_config(path=config_repositories_path)
    templates_configs = load_config(path=templates_path)
    pipeline_group_configs = load_config(path=pipeline_group_configs_path)
    elastic_agent_configs = load_config(path=elastic_agent_profile_path)
    cluster_profiles_configs = load_config(path=cluster_profiles_path)
    secret_configs = load_config(path=secret_configs_path)
    roles_configs = load_config(path=roles_path)

    response = {}
    with requests.Session() as session:
        print("Authenticate")
        authed = authenticate(session)
        if not authed:
            response["msg"] = "Failed to authenticate against: {}".format(
                GITHUB_GOCD_AUTH_URL
            )
            return False, response

        if repositories_configs:
            print("Delete Config Repositories")
            success, response = remove_configs(
                session, repositories_configs, CONFIG_REPO_URL, ACCEPT_HEADER_4
            )
            if not success:
                return False, response

        if templates_configs:
            print("Delete Template Configs")
            success, response = remove_configs(
                session,
                templates_configs,
                TEMPLATE_URL,
                ACCEPT_HEADER_7,
                identifier_variable="name",
            )
            if not success:
                return False, response

        if pipeline_group_configs:
            print("Delete Pipeline Config Groups")
            success, response = remove_configs(
                session,
                pipeline_group_configs,
                PIPELINE_GROUPS_URL,
                ACCEPT_HEADER_1,
                identifier_variable="name",
            )
            if not success:
                return False, response

        if elastic_agent_configs:
            print("Delete Elastic Agent Configs")
            success, response = remove_configs(
                session,
                elastic_agent_configs,
                ELASTIC_AGENT_URL,
                ACCEPT_HEADER_2,
            )
            if not success:
                return False, response

        if cluster_profiles_configs:
            print("Delete Cluster Profiles")
            success, response = remove_configs(
                session,
                cluster_profiles_configs,
                CLUSTER_PROFILES_URL,
                ACCEPT_HEADER_1,
            )
            if not success:
                return False, response

        if secret_configs:
            print("Delete Secret Configs")
            success, response = remove_configs(
                session,
                secret_configs,
                SECRET_CONFIG_URL,
                ACCEPT_HEADER_3,
            )
            if not success:
                return False, response

        if roles_configs:
            print("Delete Roles")
            success, response = remove_configs(
                session,
                roles_configs,
                ROLE_URL,
                ACCEPT_HEADER_3,
                identifier_variable="name",
            )
            if not success:
                return False, response

        if "msg" not in response:
            response["msg"] = "Succesfully finished the cleanup of endpoint: {}".format(
                GOCD_BASE_URL
            )
        return True, response
    return None, response


if __name__ == "__main__":
    init_server()
    configure_server()
    wipe_server()



def import_from_module(module_path, module_name, func_name):
    module = __import__(module_path, fromlist=[module_name])
    return getattr(module, func_name)


def cli_exec(args):
    # Action determines which function to execute
    module_path = args.module_path
    module_name = args.module_name
    func_name = args.func_name
    func = import_from_module(module_path, module_name, func_name)
    if not func:
        return False

    # Extract none config kwargs from args
    return func()



import os
import datetime
import sys
import subprocess
import inspect
import json
from ast import literal_eval


def get_env_value(name):
    value, msg = is_env_set(name)
    if not value:
        return False, msg
    return value


def is_env_set(name):
    if name not in os.environ:
        return False, "Environment variable: {} is not set".format(name)
    value = os.environ[name]
    if not value:
        return False, "Environment variable: {} is set but blank".format(name)
    return value, ""


def process(execute_kwargs=None):
    """Function for execute a set of command lines"""
    if not execute_kwargs:
        execute_kwargs = {}

    commands = execute_kwargs["commands"]
    if not isinstance(commands, list):
        commands = [execute_kwargs["commands"]]

    output_results = []
    for command in commands:
        if isinstance(command, str):
            prepared_command = command.split()
        elif isinstance(command, list):
            prepared_command = command
        else:
            raise TypeError("Incorrect command type handed to process")
        # Subprocess
        run_kwargs = {}
        available_arguments = inspect.getfullargspec(subprocess.run)
        if (
            "capture_output" in available_arguments.kwonlyargs
            and "capture" in execute_kwargs
        ):
            run_kwargs["capture_output"] = execute_kwargs["capture"]
        else:
            if execute_kwargs["capture"]:
                run_kwargs["stdout"] = subprocess.PIPE
                run_kwargs["stderr"] = subprocess.PIPE

        result = subprocess.run(prepared_command, **run_kwargs)
        command_results = {}
        if hasattr(result, "args"):
            command_results.update({"command": " ".join((getattr(result, "args")))})
        if hasattr(result, "returncode"):
            command_results.update({"returncode": str(getattr(result, "returncode"))})
        if hasattr(result, "stderr"):
            command_results.update({"error": str(getattr(result, "stderr"))})
        if hasattr(result, "stdout"):
            command_results.update({"output": str(getattr(result, "stdout"))})
        output_results.append(command_results)
    return output_results


def ensure_string(value):
    return_value = None
    if isinstance(value, bytes):
        return_value = value.decode("utf-8")
    if isinstance(value, str):
        # Ensure utf-8 encoding
        return_value = value.encode("utf-8")
    if isinstance(value, int):
        return_value = str(value)
    return return_value


def format_output_json(result):
    json_result = {}
    for key, value in result.items():
        try:
            evalued = literal_eval(value)
        except SyntaxError:
            # still try to encode it correctly
            string_value = ensure_string(value)
            json_result[key] = string_value
            continue

        string_evalued = ensure_string(evalued)
        if len(string_evalued) > 0:
            try:
                json_result[key] = json.loads(string_evalued)
            except Exception:
                json_result[key] = string_evalued
        else:
            json_result[key] = string_evalued
    return json_result


def eprint(*args, **kwargs):
    print(*args, file=sys.stderr, **kwargs)


def to_str(o):
    if isinstance(o, datetime.datetime):
        return o.__str__()
